#include <boost/extension/extension.hpp>
#include <boost/extension/factory.hpp>
#include <boost/extension/type_map.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>

#include <syslog.h>
#include <mysql++/mysql++.h>
#include <sys/utsname.h>

#include <plugins/plugin.hpp>
#include <ndds/ndds_cpp.h>
#include "ids_dds.h"
#include "ids_ddsSupport.h"

#include <iostream>

#ifndef SYSCONFDIR
#  define SYSCONFDIR ""
#endif

class ids_alertListener : public DDSDataReaderListener {
public:
  ids_alertListener(std::string plugin_id);
  ~ids_alertListener(void);
  
  virtual void on_requested_deadline_missed(DDSDataReader* /*reader*/,
					    const DDS_RequestedDeadlineMissedStatus& /*status*/) {}
  
  virtual void on_requested_incompatible_qos(DDSDataReader* /*reader*/,
					     const DDS_RequestedIncompatibleQosStatus& /*status*/) {}

  virtual void on_sample_rejected(DDSDataReader* /*reader*/,
				  const DDS_SampleRejectedStatus& /*status*/) {}

  virtual void on_liveliness_changed(DDSDataReader* /*reader*/,
				     const DDS_LivelinessChangedStatus& /*status*/) {}

  virtual void on_sample_lost(DDSDataReader* /*reader*/,
			      const DDS_SampleLostStatus& /*status*/) {}

  virtual void on_subscription_matched(DDSDataReader* /*reader*/,
				       const DDS_SubscriptionMatchedStatus& /*status*/) {}
  
  virtual void on_data_available(DDSDataReader* reader);

private:
  std::map<std::string,std::string> hostinfo_map_;  
  std::map<std::string,int> rowcount_map_;
  mysqlpp::Connection conn_;
  std::string database_server_;
  std::string database_name_;
  std::string database_user_;
  std::string database_password_;
  std::string plugin_id_;
};


class ids : public plugin {

public:
  ids(std::string plugin_id);
  virtual void receive_information(void);
  virtual void clean_dds_stuff(void);

private:
  //DDS stuff
  DDSDomainParticipant *participant_;
  DDSSubscriber *subscriber_;
  DDSTopic *topic_;
  ids_alertListener *reader_listener_;
  DDSDataReader *reader_;
  DDS_ReturnCode_t retcode_;
  const char *type_name_;
  DDS_Duration_t receive_period_;
  int domain_id_;
  std::string qos_profile_;
  std::string plugin_id_;

};


BOOST_EXTENSION_TYPE_MAP_FUNCTION 
{
  using namespace boost::extensions;
  std::map<std::string, factory<plugin, std::string> > &plugin_factories(types.get());
  plugin_factories["ids"].set<ids>();
}
